#!/bin/bash
# shellcheck disable=SC1091 # Cannot resolve paths in variables

PROJECT_TYPE="$1"
ACTION="$2"
PACKAGE_NAME="$3"
PROJECT_HOME="$(realpath -s --relative-to . "$(dirname "$0")/..")" || exit
PACKAGES_FILE="$PROJECT_HOME/container/packages.txt"
FLUTTER_LAUNCH="$PROJECT_HOME/swu/flutter_launch.tmpl"
PYTHON_LAUNCH="$PROJECT_HOME/swu/python_launch.tmpl"
APP_CONF="$PROJECT_HOME/config/app.conf"
APP_DIR="$PROJECT_HOME/app"
FLUTTER_YAML="$APP_DIR/pubspec.yaml"
PYTHON_REQUIREMENTS="$APP_DIR/src/requirements.txt"

. "$PROJECT_HOME/scripts/util.sh" || exit

debian_packages=()
pip_packages=()

while [ $# -gt 0 ];
do
	case "$1" in
		"--deb")
			shift
			debian_packages+=("$1") ;;
		"--pip")
			shift
			pip_packages+=("$1") ;;
		*)
			;;
	esac
	shift
done

flutter_package() {
	(
		action="$1"
		if [[ "$action" == "add" || "$action" == "remove" || "$action" == "purge" ]]; then
			if [[ "$action" == "purge" ]]; then
				action="remove"
			fi
			cd "$APP_DIR" || exit
			flutter-elinux pub "$action" "$2"
		fi
	) || error "Could not $1 $2."

	if [[ "$1" == "add" ]]; then
		for deb in "${debian_packages[@]}"; do
			if ! grep -q "$deb" "$PACKAGES_FILE"; then
				echo "$deb" >> "$PACKAGES_FILE"
			fi
		done
		if [[ "$PACKAGE_NAME" == "universal_ble" ]]; then
			sed -i "s/^\(#bluetooth\) \(.*\)/\2 \1/" \
					"$FLUTTER_LAUNCH" "$APP_CONF" \
				|| error $"Could not update config for $PACKAGE_NAME"
		fi
	elif [[ "$1" == "purge" ]]; then
		for deb in "${debian_packages[@]}"; do
			sed -i "/^$deb$/d" "$PACKAGES_FILE"
		done
		if [[ "$PACKAGE_NAME" == "universal_ble" ]]; then
			sed -i "s/^\(.*\) \(#bluetooth\)/\2 \1/" \
					"$FLUTTER_LAUNCH" "$APP_CONF" \
				|| error $"Could not update config for $PACKAGE_NAME"
		fi
	elif [[ "$1" == "status" ]]; then
		version=$(yq -r ".dependencies.$PACKAGE_NAME" "$FLUTTER_YAML")
		status="installed"
		if [[ "$version" == "null" ]]; then
			status="not installed"
			version="-"
		fi
		echo -n "$status,${version##^}"
	fi
}

python_package() {
	if [[ "$1" == "add" ]]; then
		for deb in "${debian_packages[@]}"; do
			if ! grep -q "$deb" "$PACKAGES_FILE"; then
				echo "$deb" >> "$PACKAGES_FILE"
			fi
		done
		for pip in "${pip_packages[@]}"; do
			if ! grep -q "$pip" "$PYTHON_REQUIREMENTS"; then
				echo "$pip" >> "$PYTHON_REQUIREMENTS"
			fi
		done
		if [[ "$PACKAGE_NAME" == "bleak" ]]; then
			sed -i "s/^\(#bluetooth\) \(.*\)/\2 \1/" \
					"$PYTHON_LAUNCH" "$APP_CONF" \
				|| error $"Could not update config for $PACKAGE_NAME"
		fi
	elif [[ "$1" == "remove" || "$1" == "purge" ]]; then
		for deb in "${debian_packages[@]}"; do
			sed -i "/^$deb$/d" "$PACKAGES_FILE"
		done
		for pip in "${pip_packages[@]}"; do
			sed -i "/^$pip$/d" "$PYTHON_REQUIREMENTS"
		done
		if [[ "$PACKAGE_NAME" == "bleak" ]]; then
			sed -i "s/^\(.*\) \(#bluetooth\)/\2 \1/" "$PYTHON_LAUNCH"
			sed -i "s/^\(.*\) \(#bluetooth\)/\2 \1/" "$APP_CONF"
		fi
	elif [[ "$1" == "status" ]]; then
		status="installed"
		for deb in "${debian_packages[@]}"; do
			if ! grep -q "$deb" "$PACKAGES_FILE"; then
				status="not installed"
			fi
		done
		for pip in "${pip_packages[@]}"; do
			if ! grep -q "$pip" "$PYTHON_REQUIREMENTS"; then
				status="not installed"
			fi
		done
		# the version is determined at the time of installation
		# into the container and is not displayed in the case of python
		echo -n "$status,-"
	fi
}

if [[ "$PROJECT_TYPE" == "flutter" ]]; then
	flutter_package "$ACTION" "$PACKAGE_NAME"
elif [[ "$PROJECT_TYPE" == "python" ]]; then
	python_package "$ACTION" "$PACKAGE_NAME"
fi
